from typing import List
from FreeTAKServer.components.core.abstract_component.cot_node import CoTNode
from FreeTAKServer.components.core.abstract_component.cot_property import CoTProperty
from ._mission_external_data import MissionExternalData

class MissionData(CoTNode):
    def __init__(self, configuration, model, oid=None):
        super().__init__(self.__class__.__name__, configuration, model, oid)
        self.cot_attributes["name"] = None
        self.cot_attributes["description"] = None
        self.cot_attributes["chatRoom"] = None
        self.cot_attributes["baseLayer"] = None
        self.cot_attributes["bbox"] = None
        self.cot_attributes["path"] = None
        self.cot_attributes["classification"] = None
        self.cot_attributes["tool"] = None
        self.cot_attributes["keywords"] = None
        self.cot_attributes["creatorUid"] = None
        self.cot_attributes["createTime"] = None
        self.cot_attributes["groups"] = None
        self.cot_attributes["externalData"] = []
        self.cot_attributes["feeds"] = None
        self.cot_attributes["mapLayers"] = None
        self.cot_attributes["defaultRole"] = None
        self.cot_attributes["ownerRole"] = None
        self.cot_attributes["inviteOnly"] = None
        self.cot_attributes["expiration"] = None
        self.cot_attributes["guid"] = None
        self.cot_attributes["uids"] = None
        self.cot_attributes["contents"] = []
        self.cot_attributes["token"] = None
        self.cot_attributes["passwordProtected"] = None

    @CoTProperty
    def name(self):
        return self.cot_attributes.get("name", None)
    
    @name.setter
    def name(self, name=None):
        self.cot_attributes["name"] = name
    
    @CoTProperty
    def description(self):
        return self.cot_attributes.get("description", None)
    
    @description.setter
    def description(self, description=None):
        self.cot_attributes["description"] = description

    @CoTProperty
    def chatRoom(self):
        return self.cot_attributes.get("chatRoom", None)
    
    @chatRoom.setter
    def chatRoom(self, chatRoom=None):
        self.cot_attributes["chatRoom"] = chatRoom

    @CoTProperty
    def baseLayer(self):
        return self.cot_attributes.get("baseLayer", None)
    
    @baseLayer.setter
    def baseLayer(self, baseLayer=None):
        self.cot_attributes["baseLayer"] = baseLayer
        
    @CoTProperty
    def bbox(self):
        return self.cot_attributes.get("bbox", None)
    
    @bbox.setter
    def bbox(self, bbox=None):
        self.cot_attributes["bbox"] = bbox

    @CoTProperty
    def path(self):
        return self.cot_attributes.get("path", None)
    
    @path.setter
    def path(self, path=None):
        self.cot_attributes["path"] = path

    @CoTProperty
    def classification(self):
        return self.cot_attributes.get("classification", None)
    
    @classification.setter
    def classification(self, classification=None):
        self.cot_attributes["classification"] = classification


    @CoTProperty
    def tool(self):
        return self.cot_attributes.get("tool")

    @tool.setter
    def tool(self, tool=None):
        self.cot_attributes["tool"] = tool

    @CoTProperty
    def keywords(self):
        return self.cot_attributes.get("keywords")

    @keywords.setter
    def keywords(self, keywords=None):
        self.cot_attributes["keywords"] = keywords

    @CoTProperty
    def creatorUid(self):
        return self.cot_attributes.get("creatorUid")
    
    @creatorUid.setter
    def creatorUid(self, creatorUid=None):
        self.cot_attributes["creatorUid"] = creatorUid

    @CoTProperty
    def createTime(self):
        return self.cot_attributes.get("createTime")
    
    @createTime.setter
    def createTime(self, createTime=None):
        self.cot_attributes["createTime"] = createTime

    @CoTProperty
    def groups(self):
        return self.cot_attributes.get("groups")
    
    @groups.setter
    def groups(self, groups=None):
        self.cot_attributes["groups"] = groups

    @CoTProperty
    def externalData(self) -> List[MissionExternalData]:
        return self.get_children_ex(children_type="MissionExternalData")
    
    @externalData.setter
    def externalData(self, externalData: MissionExternalData=None): # type: ignore
        self.add_child(externalData)

    @CoTProperty
    def feeds(self):
        return self.cot_attributes.get("feeds")
    
    @feeds.setter
    def feeds(self, feeds=None):
        self.cot_attributes["feeds"] = feeds

    @CoTProperty
    def mapLayers(self):
        return self.cot_attributes.get("mapLayers")
    
    @mapLayers.setter
    def mapLayers(self, mapLayers=None):
        self.cot_attributes["mapLayers"] = mapLayers

    @CoTProperty
    def inviteOnly(self):
        return self.cot_attributes.get("inviteOnly")
    
    @inviteOnly.setter
    def inviteOnly(self, inviteOnly=None):
        self.cot_attributes["inviteOnly"] = inviteOnly

    @CoTProperty
    def expiration(self):
        return self.cot_attributes.get("expiration")
    
    @expiration.setter
    def expiration(self, expiration=None):
        self.cot_attributes["expiration"] = expiration

    @CoTProperty
    def guid(self):
        return self.cot_attributes.get("guid")
    
    @guid.setter
    def guid(self, guid=None):
        self.cot_attributes["guid"] = guid

    @CoTProperty
    def uids(self):
        children: List = self.get_children_ex(children_type="MissionCoTContent")
        children.extend(self.get_children_ex(children_type="MissionListCoTContent"))
        return children
    
    @uids.setter
    def uids(self, uid=None): # type: ignore
        self.add_child(uid)

    @CoTProperty
    def passwordProtected(self):
        return self.cot_attributes.get("passwordProtected")
    
    @passwordProtected.setter
    def passwordProtected(self, passwordProtected=None):
        self.cot_attributes["passwordProtected"] = passwordProtected

    @CoTProperty
    def contents(self):
        return self.get_children_ex(children_type="MissionContent")

    @contents.setter
    def contents(self, contents):
        self.add_child(contents)

    @CoTProperty
    def defaultRole(self):
        return self.cot_attributes.get("MissionRoleDefault")
    
    @defaultRole.setter
    def defaultRole(self, defaultRole=None):
        self.cot_attributes["MissionRoleDefault"] = defaultRole
    
    @CoTProperty
    def ownerRole(self):
        return self.cot_attributes.get("MissionRoleOwner")
    
    @ownerRole.setter
    def ownerRole(self, ownerRole=None):
        self.cot_attributes["MissionRoleOwner"] = ownerRole

    @CoTProperty
    def token(self):
        return self.cot_attributes.get("token")
    
    @token.setter
    def token(self, token=None):
        self.cot_attributes["token"] = token